--
-- Reference design - Simple LEDs on Spartan-3E Starter Kit 
--
-- George Economakos - NTUA/uLab - June 2008
--
-- This design demonstrates how to interface to the push buttons and simple LEDs.
--    At the start, only one LED is on. 
--    Pressing the east and west buttons will cause the LED which is on to appear to 
--    also move in the corresponding direction.
--    Pressing the north button will invert all LEDs so that only one is off.
--
-- The design also uses the 50MHz oscillator provided on the board.
--
------------------------------------------------------------------------------------
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
------------------------------------------------------------------------------------
--
--
entity left_right_leds is
    Port (         
--TODO1--: Declare Input (buttons,clk) and Output (leds) ports 
	
	);
    end left_right_leds;
--
------------------------------------------------------------------------------------
--
-- Start of test architecture
--
architecture RTL of left_right_leds is
--
------------------------------------------------------------------------------------
--
-- Signals used to synchronize buttons
--

--TODO2--: Declare Signals that connect to buttons





--
-- Signals used to delay buttons so as to capture positive events
--

--TODO3--: Declare Signals similar with above that have suffix "_delayed"



--
-- Signal used to keep direction and inversion information
-- 00 = no movement
-- 01 = move left
-- 10 = move right
-- 11 = inversion 
--
signal         btn_ctrl : std_logic_vector(1 downto 0):= "00"; --initial value, no movement and no inversion
--
-- Signal used to track positive events
--
signal        btn_event : std_logic; 
--
-- Signals used to drive LEDs
--
signal      led_pattern : std_logic_vector(7 downto 0):= "00010000"; --initial value puts one LED on near the middle
--
-- Signal used as a hardware delay, for button debouncing
--
signal counter: natural range 0 to 2**23-1:=0; --0 every 167.68ms
--
------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
-- Start of circuit description
--
begin
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- Insert 2 flip-flops after each button.
  -- 1 for synchronization and 1 to track events.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  button_ctrl: process(clk)
  begin
    if clk'event and clk='1' then
	   btn_north_in <= btn_north;
		btn_east_in <= btn_east;
		btn_west_in <= btn_west;
		btn_north_delayed <= btn_north_in;
		btn_east_delayed <= btn_east_in;
		btn_west_delayed <= btn_west_in;
	 end if;
  end process button_ctrl;
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- Combinational circuits to encode movement direction and inversion.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  btn_ctrl <= "10" when btn_east_in='1' else 
              "01" when btn_west_in='1' else
      		  "11" when btn_north_in='1' else
  				  "00";
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- Combinational circuits to track positive input events.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  btn_event <= '1' when ((btn_north_in='1') and (btn_north_delayed='0')) or
                        ((btn_east_in='1') and (btn_east_delayed='0')) or
					         ((btn_west_in='1') and (btn_west_delayed='0')) else
					'0';
  --
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- LED control.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  -- The LED pattern is provided by a shift register which has been initialised in the signal 
  -- definition to have only one bit active (High). 
  -- In case of an event, the appropriate shift is performed.
  -- A hardware delay is initiated after each shift for button debouncing.
  --
  led_display: process(clk)
  begin
    if clk'event and clk='1' then
	   if counter/=0 then
		  counter <= counter - 1;
		end if;
      if counter=0 and btn_event='1' and btn_ctrl="01" then 
        led_pattern <= led_pattern(6 downto 0) & led_pattern(7); --rotate LEDs to left
		  counter <= 2**23-1;
      elsif counter=0 and btn_event='1' and btn_ctrl="10" then
        --TODO4--: Fix the next led_pattern line 
		led_pattern <=          --rotate LEDs to right
		  counter <= 2**23-1;
      elsif counter=0 and btn_event='1' and btn_ctrl="11" then 
        led_pattern <= led_pattern xor "11111111";
        counter <= 2**23-1;		  
		else
		  led_pattern <= led_pattern;
      end if;
	 end if;
  end process led_display;
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- Combinational circuit to drive the LEDs (direct connection).
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  --TODO5--: Transfer the value of led_pattern; to output port that drive the LEDs
 
  --
end RTL;

------------------------------------------------------------------------------------------------------------------------------------
--
-- END OF FILE left_right_leds.vhd
--
------------------------------------------------------------------------------------------------------------------------------------

